# Generated from VerilogPreParser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .VerilogPreParser import VerilogPreParser
else:
    from VerilogPreParser import VerilogPreParser

# This class defines a complete generic visitor for a parse tree produced by VerilogPreParser.

class VerilogPreParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by VerilogPreParser#source_text.
    def visitSource_text(self, ctx:VerilogPreParser.Source_textContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#compiler_directive.
    def visitCompiler_directive(self, ctx:VerilogPreParser.Compiler_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#begin_keywords_directive.
    def visitBegin_keywords_directive(self, ctx:VerilogPreParser.Begin_keywords_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#celldefine_directive.
    def visitCelldefine_directive(self, ctx:VerilogPreParser.Celldefine_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#default_nettype_directive.
    def visitDefault_nettype_directive(self, ctx:VerilogPreParser.Default_nettype_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#default_nettype_value.
    def visitDefault_nettype_value(self, ctx:VerilogPreParser.Default_nettype_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#else_directive.
    def visitElse_directive(self, ctx:VerilogPreParser.Else_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#elsif_directive.
    def visitElsif_directive(self, ctx:VerilogPreParser.Elsif_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#end_keywords_directive.
    def visitEnd_keywords_directive(self, ctx:VerilogPreParser.End_keywords_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#endcelldefine_directive.
    def visitEndcelldefine_directive(self, ctx:VerilogPreParser.Endcelldefine_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#endif_directive.
    def visitEndif_directive(self, ctx:VerilogPreParser.Endif_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#filename.
    def visitFilename(self, ctx:VerilogPreParser.FilenameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#group_of_lines.
    def visitGroup_of_lines(self, ctx:VerilogPreParser.Group_of_linesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#identifier.
    def visitIdentifier(self, ctx:VerilogPreParser.IdentifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#ifdef_directive.
    def visitIfdef_directive(self, ctx:VerilogPreParser.Ifdef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#ifndef_directive.
    def visitIfndef_directive(self, ctx:VerilogPreParser.Ifndef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#include_directive.
    def visitInclude_directive(self, ctx:VerilogPreParser.Include_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#level.
    def visitLevel(self, ctx:VerilogPreParser.LevelContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#line_directive.
    def visitLine_directive(self, ctx:VerilogPreParser.Line_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_delimiter.
    def visitMacro_delimiter(self, ctx:VerilogPreParser.Macro_delimiterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_esc_newline.
    def visitMacro_esc_newline(self, ctx:VerilogPreParser.Macro_esc_newlineContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_esc_quote.
    def visitMacro_esc_quote(self, ctx:VerilogPreParser.Macro_esc_quoteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_identifier.
    def visitMacro_identifier(self, ctx:VerilogPreParser.Macro_identifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_name.
    def visitMacro_name(self, ctx:VerilogPreParser.Macro_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_quote.
    def visitMacro_quote(self, ctx:VerilogPreParser.Macro_quoteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_text.
    def visitMacro_text(self, ctx:VerilogPreParser.Macro_textContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#macro_usage.
    def visitMacro_usage(self, ctx:VerilogPreParser.Macro_usageContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#nounconnected_drive_directive.
    def visitNounconnected_drive_directive(self, ctx:VerilogPreParser.Nounconnected_drive_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#number.
    def visitNumber(self, ctx:VerilogPreParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#pragma_directive.
    def visitPragma_directive(self, ctx:VerilogPreParser.Pragma_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#pragma_expression.
    def visitPragma_expression(self, ctx:VerilogPreParser.Pragma_expressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#pragma_keyword.
    def visitPragma_keyword(self, ctx:VerilogPreParser.Pragma_keywordContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#pragma_name.
    def visitPragma_name(self, ctx:VerilogPreParser.Pragma_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#pragma_value.
    def visitPragma_value(self, ctx:VerilogPreParser.Pragma_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#resetall_directive.
    def visitResetall_directive(self, ctx:VerilogPreParser.Resetall_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#source_text_.
    def visitSource_text_(self, ctx:VerilogPreParser.Source_text_Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#string_.
    def visitString_(self, ctx:VerilogPreParser.String_Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#text_macro_definition.
    def visitText_macro_definition(self, ctx:VerilogPreParser.Text_macro_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#text_macro_usage.
    def visitText_macro_usage(self, ctx:VerilogPreParser.Text_macro_usageContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#time_precision.
    def visitTime_precision(self, ctx:VerilogPreParser.Time_precisionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#time_unit.
    def visitTime_unit(self, ctx:VerilogPreParser.Time_unitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#timescale_directive.
    def visitTimescale_directive(self, ctx:VerilogPreParser.Timescale_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#unconnected_drive_directive.
    def visitUnconnected_drive_directive(self, ctx:VerilogPreParser.Unconnected_drive_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#unconnected_drive_value.
    def visitUnconnected_drive_value(self, ctx:VerilogPreParser.Unconnected_drive_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#undef_directive.
    def visitUndef_directive(self, ctx:VerilogPreParser.Undef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by VerilogPreParser#version_specifier.
    def visitVersion_specifier(self, ctx:VerilogPreParser.Version_specifierContext):
        return self.visitChildren(ctx)



del VerilogPreParser


from dovado_rtl.explorers.utilities.spaces import SOURCE_PATH
from dovado_rtl.explorers.utilities.tasks import ParsedProject
from dovado_rtl.parsers.utilities.parsed import MODULE_NAME_STR, PARAMETER_NAME_STR


class DesignPoint(ParsedProject):
    points: dict[SOURCE_PATH, dict[MODULE_NAME_STR, dict[PARAMETER_NAME_STR, str]]]

    def vectorized(self) -> list[int]:
        output_vector = []
        for _, module_to_parameters in self.points.items():
            for _, parameter_to_value in module_to_parameters.items():
                for _, value in parameter_to_value.items():
                    output_vector.append(int(value))
        return output_vector

    def get_sources(self) -> list[SOURCE_PATH]:
        return list(self.points.keys())

    def get_modules(self, source: SOURCE_PATH) -> list[MODULE_NAME_STR]:
        return list(self.points[source].keys())

    def get_parameters(
        self, source: SOURCE_PATH, module: MODULE_NAME_STR
    ) -> list[PARAMETER_NAME_STR]:
        return list(self.points[source][module].keys())

    def get_parameter_value(
        self,
        source: SOURCE_PATH,
        module: MODULE_NAME_STR,
        parameter: PARAMETER_NAME_STR,
    ) -> str:
        return self.points[source][module][parameter]

    @staticmethod
    def make_design_point_from_vector(
        vector: list[int],
        parameters_structure: dict[
            SOURCE_PATH, dict[MODULE_NAME_STR, list[PARAMETER_NAME_STR]]
        ],
        parsed_project: ParsedProject,
    ) -> "DesignPoint":
        points = {}
        current_vector_position = -1
        for source, module_to_parameter in parameters_structure.items():
            points[source] = {}
            for module, parameters in module_to_parameter.items():
                points[source][module] = {}
                for parameter in parameters:
                    current_vector_position += 1
                    if current_vector_position == len(vector):
                        raise ValueError(
                            "The vector provided to build the design point '"
                            + str(vector)
                            + "' is shorter than the number of available parameters."
                        )
                    points[source][module][parameter] = str(
                        vector[current_vector_position]
                    )

        if current_vector_position != len(vector) - 1:
            raise ValueError(
                "The design point vector has length "
                + str(len(vector))
                + " but only "
                + str(current_vector_position)
                + " parameters can be explored."
            )
        return DesignPoint(**(dict(parsed_project) | {"points": points}))


class EvaluatedDesignPoint(DesignPoint):
    design_value: dict[str, float]

    class Config:
        extra = "allow"



#!/usr/bin/env python3
from pathlib import Path
from typing import cast
from dovado_rtl.parsers.utilities.antlr.antlr_parser import AntlrParser
from dovado_rtl.parsers.utilities.antlr.hdl.hdl_antlr_module import HdlAntlrModule
import antlr4


class HdlAntlrParser(AntlrParser):
    def _parse(
        self, to_parse: Path
    ) -> tuple[antlr4.CommonTokenStream, tuple[HdlAntlrModule]]:
        return cast(
            tuple[antlr4.CommonTokenStream, tuple[HdlAntlrModule]],
            super()._parse(to_parse),
        )



from pathlib import Path

from dovado_rtl.parsers.vhdl.generated.vhdlParser import (
    vhdlParser as GeneratedVhdlParser,
)
from dovado_rtl.parsers.vhdl.generated.vhdlLexer import (
    vhdlLexer as GeneratedVhdlLexer,
)
from dovado_rtl.parsers.vhdl.vhdl_visitor import VhdlVisitor

from dovado_rtl.parsers.vhdl.vhdl_parsed import VhdlParsed
import antlr4

from dovado_rtl.parsers.utilities.antlr.hdl.hdl_antlr_parser import HdlAntlrParser


class VhdlParser(HdlAntlrParser):
    __grammar_top_rule: str = "design_file"

    def parse(self, to_parse: Path) -> VhdlParsed:
        token_stream, entities = self._parse(to_parse)
        return VhdlParsed(token_stream, entities)

    @property
    def _grammar_top_rule(self) -> str:
        return self.__grammar_top_rule

    @property
    def _lexer_type(self) -> type[antlr4.Lexer]:
        return GeneratedVhdlLexer

    @property
    def _parser_type(self) -> type[antlr4.Parser]:
        return GeneratedVhdlParser

    @property
    def _visitor_type(self) -> type[antlr4.ParseTreeVisitor]:
        return VhdlVisitor



from pathlib import Path

from dovado_rtl.parsers.utilities import HdlAntlrParser
from dovado_rtl.parsers.system_verilog.system_verilog_parsed import SystemVerilogParsed
from dovado_rtl.parsers.system_verilog.generated.SystemVerilogLexer import (
    SystemVerilogLexer as GeneratedSystemVerilogLexer,
)
from dovado_rtl.parsers.system_verilog.generated.SystemVerilogParser import (
    SystemVerilogParser as GeneratedSystemVerilogParser,
)
from dovado_rtl.parsers.system_verilog.system_verilog_visitor import (
    SystemVerilogVisitor,
)

import antlr4


class SystemVerilogParser(HdlAntlrParser):
    __grammar_top_rule: str = "source_text"

    def parse(self, to_parse: Path) -> SystemVerilogParsed:
        token_stream, modules = self._parse(to_parse)
        return SystemVerilogParsed(token_stream, modules)

    @property
    def _grammar_top_rule(self) -> str:
        return self.__grammar_top_rule

    @property
    def _lexer_type(self) -> type[antlr4.Lexer]:
        return GeneratedSystemVerilogLexer

    @property
    def _parser_type(self) -> type[antlr4.Parser]:
        return GeneratedSystemVerilogParser

    @property
    def _visitor_type(self) -> type[antlr4.ParseTreeVisitor]:
        return SystemVerilogVisitor



from typing import Optional, Sequence, Union, cast
from antlr4 import IllegalStateException
from dovado_rtl.parsers.utilities import (
    AntlrParameter,
    Port,
)
from dovado_rtl.parsers.verilog.generated.VerilogParserVisitor import (
    VerilogParserVisitor,
)
from dovado_rtl.parsers.verilog.generated.VerilogParser import VerilogParser
from dovado_rtl.parsers.utilities.antlr.hdl.hdl_antlr_module import HdlAntlrModule


class VerilogVisitor(VerilogParserVisitor):
    def visitSource_text(
        self, ctx: VerilogParser.Source_textContext
    ) -> Sequence[HdlAntlrModule]:
        modules: list[HdlAntlrModule]
        parsed_description: Optional[HdlAntlrModule]

        descriptions = ctx.description()
        modules = []
        if descriptions:
            for description in descriptions:
                parsed_description = self.visitDescription(description)
                if parsed_description:
                    modules.append(parsed_description)
        return modules

    def visitDescription(
        self, ctx: VerilogParser.DescriptionContext
    ) -> Optional[HdlAntlrModule]:
        module_declaration = ctx.module_declaration()

        if module_declaration:
            return self.visitModule_declaration(module_declaration)
        return None

    def visitModule_declaration(
        self, ctx: VerilogParser.Module_declarationContext
    ) -> HdlAntlrModule:
        parameters: list[AntlrParameter]
        ports: list[Port]
        parsed_module_item: Optional[Union[list[AntlrParameter], list[Port]]]

        module_identifier = ctx.module_identifier()
        module_parameter_port_list = ctx.module_parameter_port_list()
        list_of_port_declarations = ctx.list_of_port_declarations()
        module_items = ctx.module_item()

        ports = []
        parameters = []

        if module_parameter_port_list:
            parameters = self.visitModule_parameter_port_list(
                module_parameter_port_list
            )
        if list_of_port_declarations:
            ports = self.visitList_of_port_declarations(list_of_port_declarations)
        if module_items:
            for module_item in module_items:
                parsed_module_item = self.visitModule_item(module_item)
                if parsed_module_item:
                    if isinstance(parsed_module_item[0], Port):
                        parsed_module_item = cast(list[Port], parsed_module_item)
                        ports += parsed_module_item

                    if isinstance(parsed_module_item[0], AntlrParameter):
                        parsed_module_item = cast(
                            list[AntlrParameter], parsed_module_item
                        )
                        parameters += parsed_module_item

        if module_identifier:
            return HdlAntlrModule(
                name=module_identifier.getText(), parameters=parameters, ports=ports
            )
        raise IllegalStateException("A module must have a valid identifier")

    def visitModule_item(
        self, ctx: VerilogParser.Module_itemContext
    ) -> Optional[Union[list[Port], list[AntlrParameter]]]:
        # we ignore generate and specify block

        port_declaration = ctx.port_declaration()
        module_or_generate_item = ctx.module_or_generate_item()
        parameter_declaration = ctx.parameter_declaration()
        specparam_declaration = ctx.specparam_declaration()

        if port_declaration:
            # list[Port]
            return self.visitPort_declaration(port_declaration)
        if module_or_generate_item:
            # list[AntlrParameter]
            return self.visitModule_or_generate_item(module_or_generate_item)
        if parameter_declaration:
            # list[AntlrParameter]
            return self.visitParameter_declaration(parameter_declaration)
        if specparam_declaration:
            # list[AntlrParameter]
            return self.visitSpecparam_declaration(specparam_declaration)
        return None

    def visitModule_or_generate_item(
        self, ctx: VerilogParser.Module_or_generate_itemContext
    ) -> list[AntlrParameter]:
        local_parameter_declaration = ctx.local_parameter_declaration()

        if local_parameter_declaration:
            # list[AntlrParameter]
            return self.visitLocal_parameter_declaration(local_parameter_declaration)
        return []

    def visitLocal_parameter_declaration(
        self, ctx: VerilogParser.Local_parameter_declarationContext
    ) -> list[AntlrParameter]:
        list_of_param_assignments = ctx.list_of_param_assignments()

        if list_of_param_assignments:
            return self.visitList_of_param_assignments(list_of_param_assignments)
        return []

    def visitList_of_port_declarations(
        self, ctx: VerilogParser.List_of_port_declarationsContext
    ) -> list[Port]:
        # We ignore port, port_implicit and port_explicit
        # as they deal with port connection and not declaration

        parsed_ports: list[Port]

        port_declarations = ctx.port_declaration()

        if port_declarations:
            parsed_ports = []
            for port_declaration in port_declarations:
                parsed_ports += self.visitPort_declaration(port_declaration)
            return parsed_ports
        return []

    def visitPort_declaration(
        self, ctx: VerilogParser.Port_declarationContext
    ) -> list[Port]:
        inout_declaration = ctx.inout_declaration()
        input_declaration = ctx.input_declaration()
        output_declaration = ctx.output_declaration()

        if inout_declaration:
            return self.visitInout_declaration(inout_declaration)

        if input_declaration:
            return self.visitInput_declaration(input_declaration)

        if output_declaration:
            return self.visitOutput_declaration(output_declaration)

        return []

    def visitOutput_declaration(
        self, ctx: VerilogParser.Output_declarationContext
    ) -> list[Port]:
        identifiers: list[str]

        list_of_port_identifiers = ctx.list_of_port_identifiers()

        if list_of_port_identifiers:
            identifiers = self.visitList_of_port_identifiers(list_of_port_identifiers)
            return [
                Port(
                    name=identifier,
                    direction="output",
                    dimension="vectorial" if ctx.range_() else "scalar",
                    has_default=False,
                )
                for identifier in identifiers
            ]
        return []

    def visitInput_declaration(
        self, ctx: VerilogParser.Input_declarationContext
    ) -> list[Port]:
        identifiers: list[str]

        list_of_port_identifiers = ctx.list_of_port_identifiers()

        if list_of_port_identifiers:
            identifiers = self.visitList_of_port_identifiers(list_of_port_identifiers)
            return [
                Port(
                    name=identifier,
                    direction="input",
                    dimension="vectorial" if ctx.range_() else "scalar",
                    has_default=False,
                )
                for identifier in identifiers
            ]
        return []

    def visitInout_declaration(
        self, ctx: VerilogParser.Inout_declarationContext
    ) -> list[Port]:
        identifiers: list[str]

        list_of_port_identifiers = ctx.list_of_port_identifiers()

        if list_of_port_identifiers:
            identifiers = self.visitList_of_port_identifiers(list_of_port_identifiers)
            return [
                Port(
                    name=identifier,
                    direction="inout",
                    dimension="vectorial" if ctx.range_() else "scalar",
                    has_default=False,
                )
                for identifier in identifiers
            ]
        return []

    def visitList_of_port_identifiers(
        self, ctx: VerilogParser.List_of_port_identifiersContext
    ) -> list[str]:
        port_identifiers = ctx.port_identifier()

        if port_identifiers:
            return [port_identifier.getText() for port_identifier in port_identifiers]

        return []

    def visitModule_parameter_port_list(
        self, ctx: VerilogParser.Module_parameter_port_listContext
    ) -> list[AntlrParameter]:
        parameters: list[AntlrParameter]

        parameter_declarations = ctx.parameter_declaration()

        parameters = []
        if parameter_declarations:
            for parameter_declaration in parameter_declarations:
                parameters += self.visitParameter_declaration(parameter_declaration)
        return parameters

    def visitParameter_declaration(
        self, ctx: VerilogParser.Parameter_declarationContext
    ) -> list[AntlrParameter]:
        list_of_param_assignments = ctx.list_of_param_assignments()

        if list_of_param_assignments:
            return self.visitList_of_param_assignments(list_of_param_assignments)
        return []

    def visitList_of_param_assignments(
        self, ctx: VerilogParser.List_of_param_assignmentsContext
    ) -> list[AntlrParameter]:
        parameters: list[AntlrParameter]

        param_assignments = ctx.param_assignment()
        parameters = []
        if param_assignments:
            for param_assignment in param_assignments:
                parameters.append(self.visitParam_assignment(param_assignment))
        return parameters

    def visitParam_assignment(
        self, ctx: VerilogParser.Param_assignmentContext
    ) -> AntlrParameter:
        parameter_identifier = ctx.parameter_identifier()
        constant_mintypmax_expression = ctx.constant_mintypmax_expression()

        if parameter_identifier and constant_mintypmax_expression:
            return AntlrParameter(
                name=parameter_identifier.getText(),
                value=constant_mintypmax_expression.getText(),
                rule=constant_mintypmax_expression,
                has_default=True,
            )
        raise IllegalStateException(
            "Verilog parameters must have identifier and initialization"
        )



from pathlib import Path
from dovado_rtl.parsers.scala.scala_visitor import ScalaVisitor
from dovado_rtl.parsers.scala.scala_parsed import ScalaParsed
from dovado_rtl.parsers.utilities.antlr.antlr_parser import AntlrParser
import antlr4
from dovado_rtl.parsers.scala.generated.ScalaParser import (
    ScalaParser as GeneratedScalaParser,
)
from dovado_rtl.parsers.scala.generated.ScalaLexer import (
    ScalaLexer as GeneratedScalaLexer,
)


class ScalaParser(AntlrParser):
    __grammar_top_rule: str = "compilationUnit"

    def parse(self, to_parse: Path) -> ScalaParsed:
        token_stream, classes = self._parse(to_parse)
        return ScalaParsed(token_stream, classes)

    @property
    def _grammar_top_rule(self) -> str:
        return self.__grammar_top_rule

    @property
    def _lexer_type(self) -> type[antlr4.Lexer]:
        return GeneratedScalaLexer

    @property
    def _parser_type(self) -> type[antlr4.Parser]:
        return GeneratedScalaParser

    @property
    def _visitor_type(self) -> type[antlr4.ParseTreeVisitor]:
        return ScalaVisitor



# Generated from Scala.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ScalaParser import ScalaParser
else:
    from ScalaParser import ScalaParser

# This class defines a complete listener for a parse tree produced by ScalaParser.
class ScalaListener(ParseTreeListener):

    # Enter a parse tree produced by ScalaParser#literal.
    def enterLiteral(self, ctx:ScalaParser.LiteralContext):
        pass

    # Exit a parse tree produced by ScalaParser#literal.
    def exitLiteral(self, ctx:ScalaParser.LiteralContext):
        pass


    # Enter a parse tree produced by ScalaParser#qualId.
    def enterQualId(self, ctx:ScalaParser.QualIdContext):
        pass

    # Exit a parse tree produced by ScalaParser#qualId.
    def exitQualId(self, ctx:ScalaParser.QualIdContext):
        pass


    # Enter a parse tree produced by ScalaParser#ids.
    def enterIds(self, ctx:ScalaParser.IdsContext):
        pass

    # Exit a parse tree produced by ScalaParser#ids.
    def exitIds(self, ctx:ScalaParser.IdsContext):
        pass


    # Enter a parse tree produced by ScalaParser#stableId.
    def enterStableId(self, ctx:ScalaParser.StableIdContext):
        pass

    # Exit a parse tree produced by ScalaParser#stableId.
    def exitStableId(self, ctx:ScalaParser.StableIdContext):
        pass


    # Enter a parse tree produced by ScalaParser#classQualifier.
    def enterClassQualifier(self, ctx:ScalaParser.ClassQualifierContext):
        pass

    # Exit a parse tree produced by ScalaParser#classQualifier.
    def exitClassQualifier(self, ctx:ScalaParser.ClassQualifierContext):
        pass


    # Enter a parse tree produced by ScalaParser#type_.
    def enterType_(self, ctx:ScalaParser.Type_Context):
        pass

    # Exit a parse tree produced by ScalaParser#type_.
    def exitType_(self, ctx:ScalaParser.Type_Context):
        pass


    # Enter a parse tree produced by ScalaParser#functionArgTypes.
    def enterFunctionArgTypes(self, ctx:ScalaParser.FunctionArgTypesContext):
        pass

    # Exit a parse tree produced by ScalaParser#functionArgTypes.
    def exitFunctionArgTypes(self, ctx:ScalaParser.FunctionArgTypesContext):
        pass


    # Enter a parse tree produced by ScalaParser#existentialClause.
    def enterExistentialClause(self, ctx:ScalaParser.ExistentialClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#existentialClause.
    def exitExistentialClause(self, ctx:ScalaParser.ExistentialClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#existentialDcl.
    def enterExistentialDcl(self, ctx:ScalaParser.ExistentialDclContext):
        pass

    # Exit a parse tree produced by ScalaParser#existentialDcl.
    def exitExistentialDcl(self, ctx:ScalaParser.ExistentialDclContext):
        pass


    # Enter a parse tree produced by ScalaParser#infixType.
    def enterInfixType(self, ctx:ScalaParser.InfixTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#infixType.
    def exitInfixType(self, ctx:ScalaParser.InfixTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#compoundType.
    def enterCompoundType(self, ctx:ScalaParser.CompoundTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#compoundType.
    def exitCompoundType(self, ctx:ScalaParser.CompoundTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#annotType.
    def enterAnnotType(self, ctx:ScalaParser.AnnotTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#annotType.
    def exitAnnotType(self, ctx:ScalaParser.AnnotTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#simpleType.
    def enterSimpleType(self, ctx:ScalaParser.SimpleTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#simpleType.
    def exitSimpleType(self, ctx:ScalaParser.SimpleTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#typeArgs.
    def enterTypeArgs(self, ctx:ScalaParser.TypeArgsContext):
        pass

    # Exit a parse tree produced by ScalaParser#typeArgs.
    def exitTypeArgs(self, ctx:ScalaParser.TypeArgsContext):
        pass


    # Enter a parse tree produced by ScalaParser#types.
    def enterTypes(self, ctx:ScalaParser.TypesContext):
        pass

    # Exit a parse tree produced by ScalaParser#types.
    def exitTypes(self, ctx:ScalaParser.TypesContext):
        pass


    # Enter a parse tree produced by ScalaParser#refinement.
    def enterRefinement(self, ctx:ScalaParser.RefinementContext):
        pass

    # Exit a parse tree produced by ScalaParser#refinement.
    def exitRefinement(self, ctx:ScalaParser.RefinementContext):
        pass


    # Enter a parse tree produced by ScalaParser#refineStat.
    def enterRefineStat(self, ctx:ScalaParser.RefineStatContext):
        pass

    # Exit a parse tree produced by ScalaParser#refineStat.
    def exitRefineStat(self, ctx:ScalaParser.RefineStatContext):
        pass


    # Enter a parse tree produced by ScalaParser#typePat.
    def enterTypePat(self, ctx:ScalaParser.TypePatContext):
        pass

    # Exit a parse tree produced by ScalaParser#typePat.
    def exitTypePat(self, ctx:ScalaParser.TypePatContext):
        pass


    # Enter a parse tree produced by ScalaParser#ascription.
    def enterAscription(self, ctx:ScalaParser.AscriptionContext):
        pass

    # Exit a parse tree produced by ScalaParser#ascription.
    def exitAscription(self, ctx:ScalaParser.AscriptionContext):
        pass


    # Enter a parse tree produced by ScalaParser#expr.
    def enterExpr(self, ctx:ScalaParser.ExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#expr.
    def exitExpr(self, ctx:ScalaParser.ExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#expr1.
    def enterExpr1(self, ctx:ScalaParser.Expr1Context):
        pass

    # Exit a parse tree produced by ScalaParser#expr1.
    def exitExpr1(self, ctx:ScalaParser.Expr1Context):
        pass


    # Enter a parse tree produced by ScalaParser#prefixDef.
    def enterPrefixDef(self, ctx:ScalaParser.PrefixDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#prefixDef.
    def exitPrefixDef(self, ctx:ScalaParser.PrefixDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#postfixExpr.
    def enterPostfixExpr(self, ctx:ScalaParser.PostfixExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#postfixExpr.
    def exitPostfixExpr(self, ctx:ScalaParser.PostfixExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#infixExpr.
    def enterInfixExpr(self, ctx:ScalaParser.InfixExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#infixExpr.
    def exitInfixExpr(self, ctx:ScalaParser.InfixExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#prefixExpr.
    def enterPrefixExpr(self, ctx:ScalaParser.PrefixExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#prefixExpr.
    def exitPrefixExpr(self, ctx:ScalaParser.PrefixExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#simpleExpr.
    def enterSimpleExpr(self, ctx:ScalaParser.SimpleExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#simpleExpr.
    def exitSimpleExpr(self, ctx:ScalaParser.SimpleExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#simpleExpr1.
    def enterSimpleExpr1(self, ctx:ScalaParser.SimpleExpr1Context):
        pass

    # Exit a parse tree produced by ScalaParser#simpleExpr1.
    def exitSimpleExpr1(self, ctx:ScalaParser.SimpleExpr1Context):
        pass


    # Enter a parse tree produced by ScalaParser#exprs.
    def enterExprs(self, ctx:ScalaParser.ExprsContext):
        pass

    # Exit a parse tree produced by ScalaParser#exprs.
    def exitExprs(self, ctx:ScalaParser.ExprsContext):
        pass


    # Enter a parse tree produced by ScalaParser#argumentExprs.
    def enterArgumentExprs(self, ctx:ScalaParser.ArgumentExprsContext):
        pass

    # Exit a parse tree produced by ScalaParser#argumentExprs.
    def exitArgumentExprs(self, ctx:ScalaParser.ArgumentExprsContext):
        pass


    # Enter a parse tree produced by ScalaParser#args.
    def enterArgs(self, ctx:ScalaParser.ArgsContext):
        pass

    # Exit a parse tree produced by ScalaParser#args.
    def exitArgs(self, ctx:ScalaParser.ArgsContext):
        pass


    # Enter a parse tree produced by ScalaParser#blockExpr.
    def enterBlockExpr(self, ctx:ScalaParser.BlockExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#blockExpr.
    def exitBlockExpr(self, ctx:ScalaParser.BlockExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#block.
    def enterBlock(self, ctx:ScalaParser.BlockContext):
        pass

    # Exit a parse tree produced by ScalaParser#block.
    def exitBlock(self, ctx:ScalaParser.BlockContext):
        pass


    # Enter a parse tree produced by ScalaParser#blockStat.
    def enterBlockStat(self, ctx:ScalaParser.BlockStatContext):
        pass

    # Exit a parse tree produced by ScalaParser#blockStat.
    def exitBlockStat(self, ctx:ScalaParser.BlockStatContext):
        pass


    # Enter a parse tree produced by ScalaParser#resultExpr.
    def enterResultExpr(self, ctx:ScalaParser.ResultExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#resultExpr.
    def exitResultExpr(self, ctx:ScalaParser.ResultExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#enumerators.
    def enterEnumerators(self, ctx:ScalaParser.EnumeratorsContext):
        pass

    # Exit a parse tree produced by ScalaParser#enumerators.
    def exitEnumerators(self, ctx:ScalaParser.EnumeratorsContext):
        pass


    # Enter a parse tree produced by ScalaParser#generator.
    def enterGenerator(self, ctx:ScalaParser.GeneratorContext):
        pass

    # Exit a parse tree produced by ScalaParser#generator.
    def exitGenerator(self, ctx:ScalaParser.GeneratorContext):
        pass


    # Enter a parse tree produced by ScalaParser#caseClauses.
    def enterCaseClauses(self, ctx:ScalaParser.CaseClausesContext):
        pass

    # Exit a parse tree produced by ScalaParser#caseClauses.
    def exitCaseClauses(self, ctx:ScalaParser.CaseClausesContext):
        pass


    # Enter a parse tree produced by ScalaParser#caseClause.
    def enterCaseClause(self, ctx:ScalaParser.CaseClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#caseClause.
    def exitCaseClause(self, ctx:ScalaParser.CaseClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#guard_.
    def enterGuard_(self, ctx:ScalaParser.Guard_Context):
        pass

    # Exit a parse tree produced by ScalaParser#guard_.
    def exitGuard_(self, ctx:ScalaParser.Guard_Context):
        pass


    # Enter a parse tree produced by ScalaParser#pattern.
    def enterPattern(self, ctx:ScalaParser.PatternContext):
        pass

    # Exit a parse tree produced by ScalaParser#pattern.
    def exitPattern(self, ctx:ScalaParser.PatternContext):
        pass


    # Enter a parse tree produced by ScalaParser#pattern1.
    def enterPattern1(self, ctx:ScalaParser.Pattern1Context):
        pass

    # Exit a parse tree produced by ScalaParser#pattern1.
    def exitPattern1(self, ctx:ScalaParser.Pattern1Context):
        pass


    # Enter a parse tree produced by ScalaParser#pattern2.
    def enterPattern2(self, ctx:ScalaParser.Pattern2Context):
        pass

    # Exit a parse tree produced by ScalaParser#pattern2.
    def exitPattern2(self, ctx:ScalaParser.Pattern2Context):
        pass


    # Enter a parse tree produced by ScalaParser#pattern3.
    def enterPattern3(self, ctx:ScalaParser.Pattern3Context):
        pass

    # Exit a parse tree produced by ScalaParser#pattern3.
    def exitPattern3(self, ctx:ScalaParser.Pattern3Context):
        pass


    # Enter a parse tree produced by ScalaParser#simplePattern.
    def enterSimplePattern(self, ctx:ScalaParser.SimplePatternContext):
        pass

    # Exit a parse tree produced by ScalaParser#simplePattern.
    def exitSimplePattern(self, ctx:ScalaParser.SimplePatternContext):
        pass


    # Enter a parse tree produced by ScalaParser#patterns.
    def enterPatterns(self, ctx:ScalaParser.PatternsContext):
        pass

    # Exit a parse tree produced by ScalaParser#patterns.
    def exitPatterns(self, ctx:ScalaParser.PatternsContext):
        pass


    # Enter a parse tree produced by ScalaParser#typeParamClause.
    def enterTypeParamClause(self, ctx:ScalaParser.TypeParamClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#typeParamClause.
    def exitTypeParamClause(self, ctx:ScalaParser.TypeParamClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#funTypeParamClause.
    def enterFunTypeParamClause(self, ctx:ScalaParser.FunTypeParamClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#funTypeParamClause.
    def exitFunTypeParamClause(self, ctx:ScalaParser.FunTypeParamClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#variantTypeParam.
    def enterVariantTypeParam(self, ctx:ScalaParser.VariantTypeParamContext):
        pass

    # Exit a parse tree produced by ScalaParser#variantTypeParam.
    def exitVariantTypeParam(self, ctx:ScalaParser.VariantTypeParamContext):
        pass


    # Enter a parse tree produced by ScalaParser#typeParam.
    def enterTypeParam(self, ctx:ScalaParser.TypeParamContext):
        pass

    # Exit a parse tree produced by ScalaParser#typeParam.
    def exitTypeParam(self, ctx:ScalaParser.TypeParamContext):
        pass


    # Enter a parse tree produced by ScalaParser#paramClauses.
    def enterParamClauses(self, ctx:ScalaParser.ParamClausesContext):
        pass

    # Exit a parse tree produced by ScalaParser#paramClauses.
    def exitParamClauses(self, ctx:ScalaParser.ParamClausesContext):
        pass


    # Enter a parse tree produced by ScalaParser#paramClause.
    def enterParamClause(self, ctx:ScalaParser.ParamClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#paramClause.
    def exitParamClause(self, ctx:ScalaParser.ParamClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#params.
    def enterParams(self, ctx:ScalaParser.ParamsContext):
        pass

    # Exit a parse tree produced by ScalaParser#params.
    def exitParams(self, ctx:ScalaParser.ParamsContext):
        pass


    # Enter a parse tree produced by ScalaParser#param.
    def enterParam(self, ctx:ScalaParser.ParamContext):
        pass

    # Exit a parse tree produced by ScalaParser#param.
    def exitParam(self, ctx:ScalaParser.ParamContext):
        pass


    # Enter a parse tree produced by ScalaParser#paramType.
    def enterParamType(self, ctx:ScalaParser.ParamTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#paramType.
    def exitParamType(self, ctx:ScalaParser.ParamTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#classParamClauses.
    def enterClassParamClauses(self, ctx:ScalaParser.ClassParamClausesContext):
        pass

    # Exit a parse tree produced by ScalaParser#classParamClauses.
    def exitClassParamClauses(self, ctx:ScalaParser.ClassParamClausesContext):
        pass


    # Enter a parse tree produced by ScalaParser#classParamClause.
    def enterClassParamClause(self, ctx:ScalaParser.ClassParamClauseContext):
        pass

    # Exit a parse tree produced by ScalaParser#classParamClause.
    def exitClassParamClause(self, ctx:ScalaParser.ClassParamClauseContext):
        pass


    # Enter a parse tree produced by ScalaParser#classParams.
    def enterClassParams(self, ctx:ScalaParser.ClassParamsContext):
        pass

    # Exit a parse tree produced by ScalaParser#classParams.
    def exitClassParams(self, ctx:ScalaParser.ClassParamsContext):
        pass


    # Enter a parse tree produced by ScalaParser#classParam.
    def enterClassParam(self, ctx:ScalaParser.ClassParamContext):
        pass

    # Exit a parse tree produced by ScalaParser#classParam.
    def exitClassParam(self, ctx:ScalaParser.ClassParamContext):
        pass


    # Enter a parse tree produced by ScalaParser#bindings.
    def enterBindings(self, ctx:ScalaParser.BindingsContext):
        pass

    # Exit a parse tree produced by ScalaParser#bindings.
    def exitBindings(self, ctx:ScalaParser.BindingsContext):
        pass


    # Enter a parse tree produced by ScalaParser#binding.
    def enterBinding(self, ctx:ScalaParser.BindingContext):
        pass

    # Exit a parse tree produced by ScalaParser#binding.
    def exitBinding(self, ctx:ScalaParser.BindingContext):
        pass


    # Enter a parse tree produced by ScalaParser#modifier.
    def enterModifier(self, ctx:ScalaParser.ModifierContext):
        pass

    # Exit a parse tree produced by ScalaParser#modifier.
    def exitModifier(self, ctx:ScalaParser.ModifierContext):
        pass


    # Enter a parse tree produced by ScalaParser#localModifier.
    def enterLocalModifier(self, ctx:ScalaParser.LocalModifierContext):
        pass

    # Exit a parse tree produced by ScalaParser#localModifier.
    def exitLocalModifier(self, ctx:ScalaParser.LocalModifierContext):
        pass


    # Enter a parse tree produced by ScalaParser#accessModifier.
    def enterAccessModifier(self, ctx:ScalaParser.AccessModifierContext):
        pass

    # Exit a parse tree produced by ScalaParser#accessModifier.
    def exitAccessModifier(self, ctx:ScalaParser.AccessModifierContext):
        pass


    # Enter a parse tree produced by ScalaParser#accessQualifier.
    def enterAccessQualifier(self, ctx:ScalaParser.AccessQualifierContext):
        pass

    # Exit a parse tree produced by ScalaParser#accessQualifier.
    def exitAccessQualifier(self, ctx:ScalaParser.AccessQualifierContext):
        pass


    # Enter a parse tree produced by ScalaParser#annotation.
    def enterAnnotation(self, ctx:ScalaParser.AnnotationContext):
        pass

    # Exit a parse tree produced by ScalaParser#annotation.
    def exitAnnotation(self, ctx:ScalaParser.AnnotationContext):
        pass


    # Enter a parse tree produced by ScalaParser#constrAnnotation.
    def enterConstrAnnotation(self, ctx:ScalaParser.ConstrAnnotationContext):
        pass

    # Exit a parse tree produced by ScalaParser#constrAnnotation.
    def exitConstrAnnotation(self, ctx:ScalaParser.ConstrAnnotationContext):
        pass


    # Enter a parse tree produced by ScalaParser#templateBody.
    def enterTemplateBody(self, ctx:ScalaParser.TemplateBodyContext):
        pass

    # Exit a parse tree produced by ScalaParser#templateBody.
    def exitTemplateBody(self, ctx:ScalaParser.TemplateBodyContext):
        pass


    # Enter a parse tree produced by ScalaParser#templateStat.
    def enterTemplateStat(self, ctx:ScalaParser.TemplateStatContext):
        pass

    # Exit a parse tree produced by ScalaParser#templateStat.
    def exitTemplateStat(self, ctx:ScalaParser.TemplateStatContext):
        pass


    # Enter a parse tree produced by ScalaParser#selfType.
    def enterSelfType(self, ctx:ScalaParser.SelfTypeContext):
        pass

    # Exit a parse tree produced by ScalaParser#selfType.
    def exitSelfType(self, ctx:ScalaParser.SelfTypeContext):
        pass


    # Enter a parse tree produced by ScalaParser#import_.
    def enterImport_(self, ctx:ScalaParser.Import_Context):
        pass

    # Exit a parse tree produced by ScalaParser#import_.
    def exitImport_(self, ctx:ScalaParser.Import_Context):
        pass


    # Enter a parse tree produced by ScalaParser#importExpr.
    def enterImportExpr(self, ctx:ScalaParser.ImportExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#importExpr.
    def exitImportExpr(self, ctx:ScalaParser.ImportExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#importSelectors.
    def enterImportSelectors(self, ctx:ScalaParser.ImportSelectorsContext):
        pass

    # Exit a parse tree produced by ScalaParser#importSelectors.
    def exitImportSelectors(self, ctx:ScalaParser.ImportSelectorsContext):
        pass


    # Enter a parse tree produced by ScalaParser#importSelector.
    def enterImportSelector(self, ctx:ScalaParser.ImportSelectorContext):
        pass

    # Exit a parse tree produced by ScalaParser#importSelector.
    def exitImportSelector(self, ctx:ScalaParser.ImportSelectorContext):
        pass


    # Enter a parse tree produced by ScalaParser#dcl.
    def enterDcl(self, ctx:ScalaParser.DclContext):
        pass

    # Exit a parse tree produced by ScalaParser#dcl.
    def exitDcl(self, ctx:ScalaParser.DclContext):
        pass


    # Enter a parse tree produced by ScalaParser#valDcl.
    def enterValDcl(self, ctx:ScalaParser.ValDclContext):
        pass

    # Exit a parse tree produced by ScalaParser#valDcl.
    def exitValDcl(self, ctx:ScalaParser.ValDclContext):
        pass


    # Enter a parse tree produced by ScalaParser#varDcl.
    def enterVarDcl(self, ctx:ScalaParser.VarDclContext):
        pass

    # Exit a parse tree produced by ScalaParser#varDcl.
    def exitVarDcl(self, ctx:ScalaParser.VarDclContext):
        pass


    # Enter a parse tree produced by ScalaParser#funDcl.
    def enterFunDcl(self, ctx:ScalaParser.FunDclContext):
        pass

    # Exit a parse tree produced by ScalaParser#funDcl.
    def exitFunDcl(self, ctx:ScalaParser.FunDclContext):
        pass


    # Enter a parse tree produced by ScalaParser#funSig.
    def enterFunSig(self, ctx:ScalaParser.FunSigContext):
        pass

    # Exit a parse tree produced by ScalaParser#funSig.
    def exitFunSig(self, ctx:ScalaParser.FunSigContext):
        pass


    # Enter a parse tree produced by ScalaParser#typeDcl.
    def enterTypeDcl(self, ctx:ScalaParser.TypeDclContext):
        pass

    # Exit a parse tree produced by ScalaParser#typeDcl.
    def exitTypeDcl(self, ctx:ScalaParser.TypeDclContext):
        pass


    # Enter a parse tree produced by ScalaParser#patVarDef.
    def enterPatVarDef(self, ctx:ScalaParser.PatVarDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#patVarDef.
    def exitPatVarDef(self, ctx:ScalaParser.PatVarDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#def_.
    def enterDef_(self, ctx:ScalaParser.Def_Context):
        pass

    # Exit a parse tree produced by ScalaParser#def_.
    def exitDef_(self, ctx:ScalaParser.Def_Context):
        pass


    # Enter a parse tree produced by ScalaParser#patDef.
    def enterPatDef(self, ctx:ScalaParser.PatDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#patDef.
    def exitPatDef(self, ctx:ScalaParser.PatDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#varDef.
    def enterVarDef(self, ctx:ScalaParser.VarDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#varDef.
    def exitVarDef(self, ctx:ScalaParser.VarDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#funDef.
    def enterFunDef(self, ctx:ScalaParser.FunDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#funDef.
    def exitFunDef(self, ctx:ScalaParser.FunDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#typeDef.
    def enterTypeDef(self, ctx:ScalaParser.TypeDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#typeDef.
    def exitTypeDef(self, ctx:ScalaParser.TypeDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#tmplDef.
    def enterTmplDef(self, ctx:ScalaParser.TmplDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#tmplDef.
    def exitTmplDef(self, ctx:ScalaParser.TmplDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#classDef.
    def enterClassDef(self, ctx:ScalaParser.ClassDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#classDef.
    def exitClassDef(self, ctx:ScalaParser.ClassDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#traitDef.
    def enterTraitDef(self, ctx:ScalaParser.TraitDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#traitDef.
    def exitTraitDef(self, ctx:ScalaParser.TraitDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#objectDef.
    def enterObjectDef(self, ctx:ScalaParser.ObjectDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#objectDef.
    def exitObjectDef(self, ctx:ScalaParser.ObjectDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#classTemplateOpt.
    def enterClassTemplateOpt(self, ctx:ScalaParser.ClassTemplateOptContext):
        pass

    # Exit a parse tree produced by ScalaParser#classTemplateOpt.
    def exitClassTemplateOpt(self, ctx:ScalaParser.ClassTemplateOptContext):
        pass


    # Enter a parse tree produced by ScalaParser#traitTemplateOpt.
    def enterTraitTemplateOpt(self, ctx:ScalaParser.TraitTemplateOptContext):
        pass

    # Exit a parse tree produced by ScalaParser#traitTemplateOpt.
    def exitTraitTemplateOpt(self, ctx:ScalaParser.TraitTemplateOptContext):
        pass


    # Enter a parse tree produced by ScalaParser#classTemplate.
    def enterClassTemplate(self, ctx:ScalaParser.ClassTemplateContext):
        pass

    # Exit a parse tree produced by ScalaParser#classTemplate.
    def exitClassTemplate(self, ctx:ScalaParser.ClassTemplateContext):
        pass


    # Enter a parse tree produced by ScalaParser#traitTemplate.
    def enterTraitTemplate(self, ctx:ScalaParser.TraitTemplateContext):
        pass

    # Exit a parse tree produced by ScalaParser#traitTemplate.
    def exitTraitTemplate(self, ctx:ScalaParser.TraitTemplateContext):
        pass


    # Enter a parse tree produced by ScalaParser#classParents.
    def enterClassParents(self, ctx:ScalaParser.ClassParentsContext):
        pass

    # Exit a parse tree produced by ScalaParser#classParents.
    def exitClassParents(self, ctx:ScalaParser.ClassParentsContext):
        pass


    # Enter a parse tree produced by ScalaParser#traitParents.
    def enterTraitParents(self, ctx:ScalaParser.TraitParentsContext):
        pass

    # Exit a parse tree produced by ScalaParser#traitParents.
    def exitTraitParents(self, ctx:ScalaParser.TraitParentsContext):
        pass


    # Enter a parse tree produced by ScalaParser#constr.
    def enterConstr(self, ctx:ScalaParser.ConstrContext):
        pass

    # Exit a parse tree produced by ScalaParser#constr.
    def exitConstr(self, ctx:ScalaParser.ConstrContext):
        pass


    # Enter a parse tree produced by ScalaParser#earlyDefs.
    def enterEarlyDefs(self, ctx:ScalaParser.EarlyDefsContext):
        pass

    # Exit a parse tree produced by ScalaParser#earlyDefs.
    def exitEarlyDefs(self, ctx:ScalaParser.EarlyDefsContext):
        pass


    # Enter a parse tree produced by ScalaParser#earlyDef.
    def enterEarlyDef(self, ctx:ScalaParser.EarlyDefContext):
        pass

    # Exit a parse tree produced by ScalaParser#earlyDef.
    def exitEarlyDef(self, ctx:ScalaParser.EarlyDefContext):
        pass


    # Enter a parse tree produced by ScalaParser#constrExpr.
    def enterConstrExpr(self, ctx:ScalaParser.ConstrExprContext):
        pass

    # Exit a parse tree produced by ScalaParser#constrExpr.
    def exitConstrExpr(self, ctx:ScalaParser.ConstrExprContext):
        pass


    # Enter a parse tree produced by ScalaParser#constrBlock.
    def enterConstrBlock(self, ctx:ScalaParser.ConstrBlockContext):
        pass

    # Exit a parse tree produced by ScalaParser#constrBlock.
    def exitConstrBlock(self, ctx:ScalaParser.ConstrBlockContext):
        pass


    # Enter a parse tree produced by ScalaParser#selfInvocation.
    def enterSelfInvocation(self, ctx:ScalaParser.SelfInvocationContext):
        pass

    # Exit a parse tree produced by ScalaParser#selfInvocation.
    def exitSelfInvocation(self, ctx:ScalaParser.SelfInvocationContext):
        pass


    # Enter a parse tree produced by ScalaParser#topStatSeq.
    def enterTopStatSeq(self, ctx:ScalaParser.TopStatSeqContext):
        pass

    # Exit a parse tree produced by ScalaParser#topStatSeq.
    def exitTopStatSeq(self, ctx:ScalaParser.TopStatSeqContext):
        pass


    # Enter a parse tree produced by ScalaParser#topStat.
    def enterTopStat(self, ctx:ScalaParser.TopStatContext):
        pass

    # Exit a parse tree produced by ScalaParser#topStat.
    def exitTopStat(self, ctx:ScalaParser.TopStatContext):
        pass


    # Enter a parse tree produced by ScalaParser#packaging.
    def enterPackaging(self, ctx:ScalaParser.PackagingContext):
        pass

    # Exit a parse tree produced by ScalaParser#packaging.
    def exitPackaging(self, ctx:ScalaParser.PackagingContext):
        pass


    # Enter a parse tree produced by ScalaParser#packageObject.
    def enterPackageObject(self, ctx:ScalaParser.PackageObjectContext):
        pass

    # Exit a parse tree produced by ScalaParser#packageObject.
    def exitPackageObject(self, ctx:ScalaParser.PackageObjectContext):
        pass


    # Enter a parse tree produced by ScalaParser#compilationUnit.
    def enterCompilationUnit(self, ctx:ScalaParser.CompilationUnitContext):
        pass

    # Exit a parse tree produced by ScalaParser#compilationUnit.
    def exitCompilationUnit(self, ctx:ScalaParser.CompilationUnitContext):
        pass



del ScalaParser


from typing import Optional
from dovado_rtl.parsers.cpp.generated.CPP14Parser import CPP14Parser
from dovado_rtl.parsers.cpp.generated.CPP14ParserVisitor import CPP14ParserVisitor
from dovado_rtl.parsers.utilities.antlr.antlr_parameter import AntlrParameter
from dovado_rtl.parsers.utilities.antlr.antlr_module import AntlrModule


class CppVisitor(CPP14ParserVisitor):
    def visitTranslationUnit(
        self, ctx: CPP14Parser.TranslationUnitContext
    ) -> list[AntlrModule]:
        declaration_seq = ctx.declarationseq()
        if declaration_seq:
            return [self.visitDeclarationseq(declaration_seq)]
        return [AntlrModule(name="", parameters=[])]

    def visitDeclarationseq(
        self, ctx: CPP14Parser.DeclarationseqContext
    ) -> AntlrModule:
        declarations = ctx.declaration()
        if declarations is None:
            raise ValueError("A declarationseq must contain at least one declaration")

        parameters: list[AntlrParameter] = []
        for declaration in declarations:
            declaration = self.visitDeclaration(declaration)
            if declaration is not None:
                parameters.append(declaration)

        return AntlrModule(name="", parameters=parameters)

    def visitDeclaration(
        self, ctx: CPP14Parser.DeclarationContext
    ) -> Optional[AntlrParameter]:
        define = ctx.define()

        if define:
            return self.visitDefine(define)
        return None

    def visitDefine(self, ctx: CPP14Parser.DefineContext) -> Optional[AntlrParameter]:
        identifier = ctx.Identifier()
        expression = ctx.primaryExpression()

        if ctx.declarator():
            return None

        if identifier is None or expression is None:
            raise ValueError("Identifier and Expression are necessary for a define")
        return AntlrParameter(
            name=str(identifier),
            value=expression.getText(),
            rule=expression,
            has_default=True,
        )



from .utilities.parameter import Parameter
from .utilities.parsed import Parsed
from .utilities.parser import Parser
from .utilities.port import Port, PORT_DIMENSION, PORT_DIRECTION
from .utilities.antlr.antlr_parameter import AntlrParameter
from .utilities.antlr.antlr_parsed import AntlrParsed
from .utilities.antlr.antlr_parser import AntlrParser
from .utilities.antlr.antlr_module import AntlrModule
from .utilities.antlr.hdl.hdl_antlr_module import HdlAntlrModule
from .utilities.antlr.hdl.hdl_antlr_parsed import HdlAntlrParsed
from .utilities.antlr.hdl.hdl_antlr_module import HdlAntlrModule
from .utilities.antlr.hdl.hdl_antlr_parser import HdlAntlrParser



from dovado_rtl.input import Input
import shutil
from pathlib import Path
import os


def project_copy(project: Input) -> Input:
    copied_project = Path(project.work_directory, project.project_root.parts[-1])
    if copied_project.exists():
        if copied_project.is_dir():
            shutil.rmtree(copied_project)
        else:
            os.remove(copied_project)

    shutil.copytree(project.project_root, copied_project, dirs_exist_ok=False)
    return Input(
        **{k: v for k, v in dict(project).items() if k != "project_root"},
        project_root=copied_project
    )



from dovado_rtl.parsers.utilities.antlr.hdl.hdl_antlr_parsed import HdlAntlrParsed


class SystemVerilogParsed(HdlAntlrParsed):
    __parameter_initialization_prefix = "="

    @property
    def _parameter_initialization_prefix(self) -> str:
        return self.__parameter_initialization_prefix



from dovado_rtl.explorers.utilities.tasks import ParsedProject
from dovado_rtl.input import Input
from dovado_rtl.parsers.parse import parse as general_parse
from dovado_rtl.parsers.scala.scala_parser import ScalaParser
from dovado_rtl.explorers.utilities.design_points import DesignPoint
from typing import Union


def parse(
    input_project: Union[Input, DesignPoint],
) -> Union[ParsedProject, DesignPoint]:
    return general_parse(input_project, ScalaParser)



from typing import Literal, Union

SYNTHESIS_DIRECTIVE = Union[
    Literal["default"],
    Literal["RuntimeOptimized"],
    Literal["AreaOptimized_high"],
    Literal["AreaOptimized_medium"],
    Literal["AlternateRoutability"],
    Literal["AreaMapLargeShiftRegToBRAM"],
    Literal["AreaMultThresholdDSP"],
    Literal["FewerCarryChains"],
    Literal["PerformanceOptimized"],
    Literal["LogicCompaction"],
    Literal["PowerOptimized_high"],
    Literal["PowerOptimized_medium"],
]
PLACE_DIRECTIVE = Union[
    Literal["Explore"],
    Literal["EarlyBlockPlacement"],
    Literal["WLDrivenBlockPlacement"],
    Literal["ExtraNetDelay_high"],
    Literal["ExtraNetDelay_low"],
    Literal["AltSpreadLogic_high"],
    Literal["AltSpreadLogic_medium"],
    Literal["AltSpreadLogic_low"],
    Literal["ExtraPostPlacementOpt"],
    Literal["ExtraTimingOpt"],
    Literal["SSI_SpreadLogic_high"],
    Literal["SSI_SpreadLogic_low"],
    Literal["SSI_SpreadSLLs"],
    Literal["SSI_BalanceSLLs"],
    Literal["SSI_BalanceSLRs"],
    Literal["SSI_HighUtilSLRs"],
    Literal["RuntimeOptimized"],
    Literal["Quick"],
    Literal["RQS"],
    Literal["Auto_1"],
    Literal["Auto_2"],
    Literal["Auto_3"],
]
ROUTE_DIRECTIVE = Union[
    Literal["Explore"],
    Literal["AggressiveExplore"],
    Literal["NoTimingRelaxation"],
    Literal["MoreGlobalIterations"],
    Literal["HigherDelayCost"],
    Literal["AdvancedSkewModeling"],
    Literal["AlternateCLBRouting"],
    Literal["RuntimeOptimized"],
    Literal["Quick"],
    Literal["Default"],
]



# Generated from SystemVerilogPreParser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .SystemVerilogPreParser import SystemVerilogPreParser
else:
    from SystemVerilogPreParser import SystemVerilogPreParser

# This class defines a complete generic visitor for a parse tree produced by SystemVerilogPreParser.

class SystemVerilogPreParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by SystemVerilogPreParser#source_text.
    def visitSource_text(self, ctx:SystemVerilogPreParser.Source_textContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#compiler_directive.
    def visitCompiler_directive(self, ctx:SystemVerilogPreParser.Compiler_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#begin_keywords_directive.
    def visitBegin_keywords_directive(self, ctx:SystemVerilogPreParser.Begin_keywords_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#celldefine_directive.
    def visitCelldefine_directive(self, ctx:SystemVerilogPreParser.Celldefine_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#default_nettype_directive.
    def visitDefault_nettype_directive(self, ctx:SystemVerilogPreParser.Default_nettype_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#default_nettype_value.
    def visitDefault_nettype_value(self, ctx:SystemVerilogPreParser.Default_nettype_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#else_directive.
    def visitElse_directive(self, ctx:SystemVerilogPreParser.Else_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#elsif_directive.
    def visitElsif_directive(self, ctx:SystemVerilogPreParser.Elsif_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#end_keywords_directive.
    def visitEnd_keywords_directive(self, ctx:SystemVerilogPreParser.End_keywords_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#endcelldefine_directive.
    def visitEndcelldefine_directive(self, ctx:SystemVerilogPreParser.Endcelldefine_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#endif_directive.
    def visitEndif_directive(self, ctx:SystemVerilogPreParser.Endif_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#file_directive.
    def visitFile_directive(self, ctx:SystemVerilogPreParser.File_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#filename.
    def visitFilename(self, ctx:SystemVerilogPreParser.FilenameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#group_of_lines.
    def visitGroup_of_lines(self, ctx:SystemVerilogPreParser.Group_of_linesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#identifier.
    def visitIdentifier(self, ctx:SystemVerilogPreParser.IdentifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#ifdef_directive.
    def visitIfdef_directive(self, ctx:SystemVerilogPreParser.Ifdef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#ifndef_directive.
    def visitIfndef_directive(self, ctx:SystemVerilogPreParser.Ifndef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#include_directive.
    def visitInclude_directive(self, ctx:SystemVerilogPreParser.Include_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#level.
    def visitLevel(self, ctx:SystemVerilogPreParser.LevelContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#line_directive.
    def visitLine_directive(self, ctx:SystemVerilogPreParser.Line_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#line_directive_.
    def visitLine_directive_(self, ctx:SystemVerilogPreParser.Line_directive_Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_delimiter.
    def visitMacro_delimiter(self, ctx:SystemVerilogPreParser.Macro_delimiterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_esc_newline.
    def visitMacro_esc_newline(self, ctx:SystemVerilogPreParser.Macro_esc_newlineContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_esc_quote.
    def visitMacro_esc_quote(self, ctx:SystemVerilogPreParser.Macro_esc_quoteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_identifier.
    def visitMacro_identifier(self, ctx:SystemVerilogPreParser.Macro_identifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_name.
    def visitMacro_name(self, ctx:SystemVerilogPreParser.Macro_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_quote.
    def visitMacro_quote(self, ctx:SystemVerilogPreParser.Macro_quoteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_text.
    def visitMacro_text(self, ctx:SystemVerilogPreParser.Macro_textContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#macro_usage.
    def visitMacro_usage(self, ctx:SystemVerilogPreParser.Macro_usageContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#nounconnected_drive_directive.
    def visitNounconnected_drive_directive(self, ctx:SystemVerilogPreParser.Nounconnected_drive_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#number.
    def visitNumber(self, ctx:SystemVerilogPreParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#pragma_directive.
    def visitPragma_directive(self, ctx:SystemVerilogPreParser.Pragma_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#pragma_expression.
    def visitPragma_expression(self, ctx:SystemVerilogPreParser.Pragma_expressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#pragma_keyword.
    def visitPragma_keyword(self, ctx:SystemVerilogPreParser.Pragma_keywordContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#pragma_name.
    def visitPragma_name(self, ctx:SystemVerilogPreParser.Pragma_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#pragma_value.
    def visitPragma_value(self, ctx:SystemVerilogPreParser.Pragma_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#resetall_directive.
    def visitResetall_directive(self, ctx:SystemVerilogPreParser.Resetall_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#source_text_.
    def visitSource_text_(self, ctx:SystemVerilogPreParser.Source_text_Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#string_literal.
    def visitString_literal(self, ctx:SystemVerilogPreParser.String_literalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#text_macro_definition.
    def visitText_macro_definition(self, ctx:SystemVerilogPreParser.Text_macro_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#text_macro_usage.
    def visitText_macro_usage(self, ctx:SystemVerilogPreParser.Text_macro_usageContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#time_precision.
    def visitTime_precision(self, ctx:SystemVerilogPreParser.Time_precisionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#time_unit.
    def visitTime_unit(self, ctx:SystemVerilogPreParser.Time_unitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#timescale_directive.
    def visitTimescale_directive(self, ctx:SystemVerilogPreParser.Timescale_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#unconnected_drive_directive.
    def visitUnconnected_drive_directive(self, ctx:SystemVerilogPreParser.Unconnected_drive_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#unconnected_drive_value.
    def visitUnconnected_drive_value(self, ctx:SystemVerilogPreParser.Unconnected_drive_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#undef_directive.
    def visitUndef_directive(self, ctx:SystemVerilogPreParser.Undef_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#undefineall_directive.
    def visitUndefineall_directive(self, ctx:SystemVerilogPreParser.Undefineall_directiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SystemVerilogPreParser#version_specifier.
    def visitVersion_specifier(self, ctx:SystemVerilogPreParser.Version_specifierContext):
        return self.visitChildren(ctx)



del SystemVerilogPreParser


from pathlib import Path
from dovado_rtl.input import Input
from dovado_rtl.project_copy import project_copy
import sys
from nacolla.parsing import parse_implementation_map, IMPLEMENTATION_MAP
from nacolla.parsing.parse_flow import parse_flow
from nacolla.flow import Flow
from importlib.resources import files
import json

_DEFAULT_STEP_FILE_NAME = "default_steps.json"


def main() -> None:

    input_project = Input.make_from_file(_parse_config_path())

    if not input_project.in_place:
        input_project = project_copy(input_project)

    implementation_map = _make_implementation_map(Path(input_project.work_directory))
    flow = make_flow(implementation_map, input_project)

    for _ in flow:
        ...


def make_flow(
    implementation_map: IMPLEMENTATION_MAP,
    input_project: Input,
) -> Flow:

    flow_name = input_project.flow
    work_directory = Path(input_project.work_directory)

    default_flow_path = Path(
        str(files("dovado_rtl.nacolla.flows").joinpath(flow_name + ".toml"))
    )

    custom_flow_path = Path(work_directory, flow_name)

    if default_flow_path.is_file():
        flow_path = default_flow_path

    elif custom_flow_path.is_file():
        flow_path = custom_flow_path

    else:
        raise ValueError("Flow '" + flow_name + "' cannot be found")

    return parse_flow(flow_path, implementation_map, source=input_project)


def _make_implementation_map(work_directory: Path) -> IMPLEMENTATION_MAP:
    default_implementation_map_path: Path = Path(
        str(files("dovado_rtl.nacolla").joinpath("steps.json"))
    )
    steps = json.load(default_implementation_map_path.open())

    fixed_steps = steps.copy()
    for index, step in enumerate(steps["implementations"]):
        step_module = step["callable"]["module"]
        step_file = step_module.split("/")[-1]
        step_path = ".".join(step_module.split("/")[:-1])
        fixed_steps["implementations"][index]["callable"]["module"] = str(
            files("dovado_rtl." + step_path).joinpath(step_file)
        )

    default_steps = Path(work_directory, _DEFAULT_STEP_FILE_NAME)
    default_steps.touch()

    with default_steps.open("w") as f:
        json.dump(fixed_steps, f)

    implementation_map = parse_implementation_map(default_steps)

    user_steps_path = Path(work_directory, "steps.json")
    if user_steps_path.is_file():
        implementation_map |= parse_implementation_map(user_steps_path)

    return implementation_map


def _parse_config_path() -> Path:
    if len(sys.argv) == 1:
        raise ValueError("Missing the path to the configuration file")
    return Path(sys.argv[1])



from abc import ABC, abstractmethod
from dovado_rtl.explorers.utilities.design_points import (
    DesignPoint,
    EvaluatedDesignPoint,
)
from pathlib import Path
from dovado_rtl.fill import fill
from typing import Union
from dovado_rtl.explorers.utilities.tasks import EndExploration
from nacolla.stateful_callable import StatefulCallable


class Builder(StatefulCallable, ABC):
    _placeholder = "____"

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def build(
        self, design_point: DesignPoint
    ) -> Union[EvaluatedDesignPoint, EndExploration]:
        ...

    def _fill(
        self,
        replacements: list[str],
        package: str,
        resource: str,
        out_path: Path,
    ) -> None:
        fill(replacements, package, resource, out_path, self._placeholder)



# Generated from CPP14Parser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .CPP14Parser import CPP14Parser
else:
    from CPP14Parser import CPP14Parser

# This class defines a complete generic visitor for a parse tree produced by CPP14Parser.

class CPP14ParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by CPP14Parser#translationUnit.
    def visitTranslationUnit(self, ctx:CPP14Parser.TranslationUnitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#primaryExpression.
    def visitPrimaryExpression(self, ctx:CPP14Parser.PrimaryExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#idExpression.
    def visitIdExpression(self, ctx:CPP14Parser.IdExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#unqualifiedId.
    def visitUnqualifiedId(self, ctx:CPP14Parser.UnqualifiedIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#qualifiedId.
    def visitQualifiedId(self, ctx:CPP14Parser.QualifiedIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#nestedNameSpecifier.
    def visitNestedNameSpecifier(self, ctx:CPP14Parser.NestedNameSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#lambdaExpression.
    def visitLambdaExpression(self, ctx:CPP14Parser.LambdaExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#lambdaIntroducer.
    def visitLambdaIntroducer(self, ctx:CPP14Parser.LambdaIntroducerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#lambdaCapture.
    def visitLambdaCapture(self, ctx:CPP14Parser.LambdaCaptureContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#captureDefault.
    def visitCaptureDefault(self, ctx:CPP14Parser.CaptureDefaultContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#captureList.
    def visitCaptureList(self, ctx:CPP14Parser.CaptureListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#capture.
    def visitCapture(self, ctx:CPP14Parser.CaptureContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleCapture.
    def visitSimpleCapture(self, ctx:CPP14Parser.SimpleCaptureContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initcapture.
    def visitInitcapture(self, ctx:CPP14Parser.InitcaptureContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#lambdaDeclarator.
    def visitLambdaDeclarator(self, ctx:CPP14Parser.LambdaDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#postfixExpression.
    def visitPostfixExpression(self, ctx:CPP14Parser.PostfixExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeIdOfTheTypeId.
    def visitTypeIdOfTheTypeId(self, ctx:CPP14Parser.TypeIdOfTheTypeIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#expressionList.
    def visitExpressionList(self, ctx:CPP14Parser.ExpressionListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pseudoDestructorName.
    def visitPseudoDestructorName(self, ctx:CPP14Parser.PseudoDestructorNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#unaryExpression.
    def visitUnaryExpression(self, ctx:CPP14Parser.UnaryExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#unaryOperator.
    def visitUnaryOperator(self, ctx:CPP14Parser.UnaryOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#newExpression.
    def visitNewExpression(self, ctx:CPP14Parser.NewExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#newPlacement.
    def visitNewPlacement(self, ctx:CPP14Parser.NewPlacementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#newTypeId.
    def visitNewTypeId(self, ctx:CPP14Parser.NewTypeIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#newDeclarator.
    def visitNewDeclarator(self, ctx:CPP14Parser.NewDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noPointerNewDeclarator.
    def visitNoPointerNewDeclarator(self, ctx:CPP14Parser.NoPointerNewDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#newInitializer.
    def visitNewInitializer(self, ctx:CPP14Parser.NewInitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#deleteExpression.
    def visitDeleteExpression(self, ctx:CPP14Parser.DeleteExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noExceptExpression.
    def visitNoExceptExpression(self, ctx:CPP14Parser.NoExceptExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#castExpression.
    def visitCastExpression(self, ctx:CPP14Parser.CastExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pointerMemberExpression.
    def visitPointerMemberExpression(self, ctx:CPP14Parser.PointerMemberExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#multiplicativeExpression.
    def visitMultiplicativeExpression(self, ctx:CPP14Parser.MultiplicativeExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#additiveExpression.
    def visitAdditiveExpression(self, ctx:CPP14Parser.AdditiveExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#shiftExpression.
    def visitShiftExpression(self, ctx:CPP14Parser.ShiftExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#shiftOperator.
    def visitShiftOperator(self, ctx:CPP14Parser.ShiftOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#relationalExpression.
    def visitRelationalExpression(self, ctx:CPP14Parser.RelationalExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#equalityExpression.
    def visitEqualityExpression(self, ctx:CPP14Parser.EqualityExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#andExpression.
    def visitAndExpression(self, ctx:CPP14Parser.AndExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#exclusiveOrExpression.
    def visitExclusiveOrExpression(self, ctx:CPP14Parser.ExclusiveOrExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#inclusiveOrExpression.
    def visitInclusiveOrExpression(self, ctx:CPP14Parser.InclusiveOrExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#logicalAndExpression.
    def visitLogicalAndExpression(self, ctx:CPP14Parser.LogicalAndExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#logicalOrExpression.
    def visitLogicalOrExpression(self, ctx:CPP14Parser.LogicalOrExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#conditionalExpression.
    def visitConditionalExpression(self, ctx:CPP14Parser.ConditionalExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#assignmentExpression.
    def visitAssignmentExpression(self, ctx:CPP14Parser.AssignmentExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#assignmentOperator.
    def visitAssignmentOperator(self, ctx:CPP14Parser.AssignmentOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#expression.
    def visitExpression(self, ctx:CPP14Parser.ExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#constantExpression.
    def visitConstantExpression(self, ctx:CPP14Parser.ConstantExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#statement.
    def visitStatement(self, ctx:CPP14Parser.StatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#labeledStatement.
    def visitLabeledStatement(self, ctx:CPP14Parser.LabeledStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#expressionStatement.
    def visitExpressionStatement(self, ctx:CPP14Parser.ExpressionStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#compoundStatement.
    def visitCompoundStatement(self, ctx:CPP14Parser.CompoundStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#statementSeq.
    def visitStatementSeq(self, ctx:CPP14Parser.StatementSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#selectionStatement.
    def visitSelectionStatement(self, ctx:CPP14Parser.SelectionStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#condition.
    def visitCondition(self, ctx:CPP14Parser.ConditionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#iterationStatement.
    def visitIterationStatement(self, ctx:CPP14Parser.IterationStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#forInitStatement.
    def visitForInitStatement(self, ctx:CPP14Parser.ForInitStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#forRangeDeclaration.
    def visitForRangeDeclaration(self, ctx:CPP14Parser.ForRangeDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#forRangeInitializer.
    def visitForRangeInitializer(self, ctx:CPP14Parser.ForRangeInitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#jumpStatement.
    def visitJumpStatement(self, ctx:CPP14Parser.JumpStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declarationStatement.
    def visitDeclarationStatement(self, ctx:CPP14Parser.DeclarationStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declarationseq.
    def visitDeclarationseq(self, ctx:CPP14Parser.DeclarationseqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declaration.
    def visitDeclaration(self, ctx:CPP14Parser.DeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#blockDeclaration.
    def visitBlockDeclaration(self, ctx:CPP14Parser.BlockDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#aliasDeclaration.
    def visitAliasDeclaration(self, ctx:CPP14Parser.AliasDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleDeclaration.
    def visitSimpleDeclaration(self, ctx:CPP14Parser.SimpleDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#staticAssertDeclaration.
    def visitStaticAssertDeclaration(self, ctx:CPP14Parser.StaticAssertDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#emptyDeclaration.
    def visitEmptyDeclaration(self, ctx:CPP14Parser.EmptyDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeDeclaration.
    def visitAttributeDeclaration(self, ctx:CPP14Parser.AttributeDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declSpecifier.
    def visitDeclSpecifier(self, ctx:CPP14Parser.DeclSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declSpecifierSeq.
    def visitDeclSpecifierSeq(self, ctx:CPP14Parser.DeclSpecifierSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#storageClassSpecifier.
    def visitStorageClassSpecifier(self, ctx:CPP14Parser.StorageClassSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#functionSpecifier.
    def visitFunctionSpecifier(self, ctx:CPP14Parser.FunctionSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typedefName.
    def visitTypedefName(self, ctx:CPP14Parser.TypedefNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeSpecifier.
    def visitTypeSpecifier(self, ctx:CPP14Parser.TypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#trailingTypeSpecifier.
    def visitTrailingTypeSpecifier(self, ctx:CPP14Parser.TrailingTypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeSpecifierSeq.
    def visitTypeSpecifierSeq(self, ctx:CPP14Parser.TypeSpecifierSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#trailingTypeSpecifierSeq.
    def visitTrailingTypeSpecifierSeq(self, ctx:CPP14Parser.TrailingTypeSpecifierSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleTypeLengthModifier.
    def visitSimpleTypeLengthModifier(self, ctx:CPP14Parser.SimpleTypeLengthModifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleTypeSignednessModifier.
    def visitSimpleTypeSignednessModifier(self, ctx:CPP14Parser.SimpleTypeSignednessModifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleTypeSpecifier.
    def visitSimpleTypeSpecifier(self, ctx:CPP14Parser.SimpleTypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#theTypeName.
    def visitTheTypeName(self, ctx:CPP14Parser.TheTypeNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#decltypeSpecifier.
    def visitDecltypeSpecifier(self, ctx:CPP14Parser.DecltypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#elaboratedTypeSpecifier.
    def visitElaboratedTypeSpecifier(self, ctx:CPP14Parser.ElaboratedTypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumName.
    def visitEnumName(self, ctx:CPP14Parser.EnumNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumSpecifier.
    def visitEnumSpecifier(self, ctx:CPP14Parser.EnumSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumHead.
    def visitEnumHead(self, ctx:CPP14Parser.EnumHeadContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#opaqueEnumDeclaration.
    def visitOpaqueEnumDeclaration(self, ctx:CPP14Parser.OpaqueEnumDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumkey.
    def visitEnumkey(self, ctx:CPP14Parser.EnumkeyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumbase.
    def visitEnumbase(self, ctx:CPP14Parser.EnumbaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumeratorList.
    def visitEnumeratorList(self, ctx:CPP14Parser.EnumeratorListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumeratorDefinition.
    def visitEnumeratorDefinition(self, ctx:CPP14Parser.EnumeratorDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#enumerator.
    def visitEnumerator(self, ctx:CPP14Parser.EnumeratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#namespaceName.
    def visitNamespaceName(self, ctx:CPP14Parser.NamespaceNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#originalNamespaceName.
    def visitOriginalNamespaceName(self, ctx:CPP14Parser.OriginalNamespaceNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#namespaceDefinition.
    def visitNamespaceDefinition(self, ctx:CPP14Parser.NamespaceDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#namespaceAlias.
    def visitNamespaceAlias(self, ctx:CPP14Parser.NamespaceAliasContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#namespaceAliasDefinition.
    def visitNamespaceAliasDefinition(self, ctx:CPP14Parser.NamespaceAliasDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#qualifiednamespacespecifier.
    def visitQualifiednamespacespecifier(self, ctx:CPP14Parser.QualifiednamespacespecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#usingDeclaration.
    def visitUsingDeclaration(self, ctx:CPP14Parser.UsingDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#usingDirective.
    def visitUsingDirective(self, ctx:CPP14Parser.UsingDirectiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#asmDefinition.
    def visitAsmDefinition(self, ctx:CPP14Parser.AsmDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#linkageSpecification.
    def visitLinkageSpecification(self, ctx:CPP14Parser.LinkageSpecificationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeSpecifierSeq.
    def visitAttributeSpecifierSeq(self, ctx:CPP14Parser.AttributeSpecifierSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeSpecifier.
    def visitAttributeSpecifier(self, ctx:CPP14Parser.AttributeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#alignmentspecifier.
    def visitAlignmentspecifier(self, ctx:CPP14Parser.AlignmentspecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeList.
    def visitAttributeList(self, ctx:CPP14Parser.AttributeListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attribute.
    def visitAttribute(self, ctx:CPP14Parser.AttributeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeNamespace.
    def visitAttributeNamespace(self, ctx:CPP14Parser.AttributeNamespaceContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#attributeArgumentClause.
    def visitAttributeArgumentClause(self, ctx:CPP14Parser.AttributeArgumentClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#balancedTokenSeq.
    def visitBalancedTokenSeq(self, ctx:CPP14Parser.BalancedTokenSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#balancedtoken.
    def visitBalancedtoken(self, ctx:CPP14Parser.BalancedtokenContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initDeclaratorList.
    def visitInitDeclaratorList(self, ctx:CPP14Parser.InitDeclaratorListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initDeclarator.
    def visitInitDeclarator(self, ctx:CPP14Parser.InitDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declarator.
    def visitDeclarator(self, ctx:CPP14Parser.DeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pointerDeclarator.
    def visitPointerDeclarator(self, ctx:CPP14Parser.PointerDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noPointerDeclarator.
    def visitNoPointerDeclarator(self, ctx:CPP14Parser.NoPointerDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#parametersAndQualifiers.
    def visitParametersAndQualifiers(self, ctx:CPP14Parser.ParametersAndQualifiersContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#trailingReturnType.
    def visitTrailingReturnType(self, ctx:CPP14Parser.TrailingReturnTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pointerOperator.
    def visitPointerOperator(self, ctx:CPP14Parser.PointerOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#cvqualifierseq.
    def visitCvqualifierseq(self, ctx:CPP14Parser.CvqualifierseqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#cvQualifier.
    def visitCvQualifier(self, ctx:CPP14Parser.CvQualifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#refqualifier.
    def visitRefqualifier(self, ctx:CPP14Parser.RefqualifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#declaratorid.
    def visitDeclaratorid(self, ctx:CPP14Parser.DeclaratoridContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#theTypeId.
    def visitTheTypeId(self, ctx:CPP14Parser.TheTypeIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#abstractDeclarator.
    def visitAbstractDeclarator(self, ctx:CPP14Parser.AbstractDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pointerAbstractDeclarator.
    def visitPointerAbstractDeclarator(self, ctx:CPP14Parser.PointerAbstractDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noPointerAbstractDeclarator.
    def visitNoPointerAbstractDeclarator(self, ctx:CPP14Parser.NoPointerAbstractDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#abstractPackDeclarator.
    def visitAbstractPackDeclarator(self, ctx:CPP14Parser.AbstractPackDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noPointerAbstractPackDeclarator.
    def visitNoPointerAbstractPackDeclarator(self, ctx:CPP14Parser.NoPointerAbstractPackDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#parameterDeclarationClause.
    def visitParameterDeclarationClause(self, ctx:CPP14Parser.ParameterDeclarationClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#parameterDeclarationList.
    def visitParameterDeclarationList(self, ctx:CPP14Parser.ParameterDeclarationListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#parameterDeclaration.
    def visitParameterDeclaration(self, ctx:CPP14Parser.ParameterDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#functionDefinition.
    def visitFunctionDefinition(self, ctx:CPP14Parser.FunctionDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#functionBody.
    def visitFunctionBody(self, ctx:CPP14Parser.FunctionBodyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initializer.
    def visitInitializer(self, ctx:CPP14Parser.InitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#braceOrEqualInitializer.
    def visitBraceOrEqualInitializer(self, ctx:CPP14Parser.BraceOrEqualInitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initializerClause.
    def visitInitializerClause(self, ctx:CPP14Parser.InitializerClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#initializerList.
    def visitInitializerList(self, ctx:CPP14Parser.InitializerListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#bracedInitList.
    def visitBracedInitList(self, ctx:CPP14Parser.BracedInitListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#className.
    def visitClassName(self, ctx:CPP14Parser.ClassNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classSpecifier.
    def visitClassSpecifier(self, ctx:CPP14Parser.ClassSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classHead.
    def visitClassHead(self, ctx:CPP14Parser.ClassHeadContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classHeadName.
    def visitClassHeadName(self, ctx:CPP14Parser.ClassHeadNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classVirtSpecifier.
    def visitClassVirtSpecifier(self, ctx:CPP14Parser.ClassVirtSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classKey.
    def visitClassKey(self, ctx:CPP14Parser.ClassKeyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memberSpecification.
    def visitMemberSpecification(self, ctx:CPP14Parser.MemberSpecificationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memberdeclaration.
    def visitMemberdeclaration(self, ctx:CPP14Parser.MemberdeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memberDeclaratorList.
    def visitMemberDeclaratorList(self, ctx:CPP14Parser.MemberDeclaratorListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memberDeclarator.
    def visitMemberDeclarator(self, ctx:CPP14Parser.MemberDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#virtualSpecifierSeq.
    def visitVirtualSpecifierSeq(self, ctx:CPP14Parser.VirtualSpecifierSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#virtualSpecifier.
    def visitVirtualSpecifier(self, ctx:CPP14Parser.VirtualSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#pureSpecifier.
    def visitPureSpecifier(self, ctx:CPP14Parser.PureSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#baseClause.
    def visitBaseClause(self, ctx:CPP14Parser.BaseClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#baseSpecifierList.
    def visitBaseSpecifierList(self, ctx:CPP14Parser.BaseSpecifierListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#baseSpecifier.
    def visitBaseSpecifier(self, ctx:CPP14Parser.BaseSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#classOrDeclType.
    def visitClassOrDeclType(self, ctx:CPP14Parser.ClassOrDeclTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#baseTypeSpecifier.
    def visitBaseTypeSpecifier(self, ctx:CPP14Parser.BaseTypeSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#accessSpecifier.
    def visitAccessSpecifier(self, ctx:CPP14Parser.AccessSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#conversionFunctionId.
    def visitConversionFunctionId(self, ctx:CPP14Parser.ConversionFunctionIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#conversionTypeId.
    def visitConversionTypeId(self, ctx:CPP14Parser.ConversionTypeIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#conversionDeclarator.
    def visitConversionDeclarator(self, ctx:CPP14Parser.ConversionDeclaratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#constructorInitializer.
    def visitConstructorInitializer(self, ctx:CPP14Parser.ConstructorInitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memInitializerList.
    def visitMemInitializerList(self, ctx:CPP14Parser.MemInitializerListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#memInitializer.
    def visitMemInitializer(self, ctx:CPP14Parser.MemInitializerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#meminitializerid.
    def visitMeminitializerid(self, ctx:CPP14Parser.MeminitializeridContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#operatorFunctionId.
    def visitOperatorFunctionId(self, ctx:CPP14Parser.OperatorFunctionIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#literalOperatorId.
    def visitLiteralOperatorId(self, ctx:CPP14Parser.LiteralOperatorIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateDeclaration.
    def visitTemplateDeclaration(self, ctx:CPP14Parser.TemplateDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateparameterList.
    def visitTemplateparameterList(self, ctx:CPP14Parser.TemplateparameterListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateParameter.
    def visitTemplateParameter(self, ctx:CPP14Parser.TemplateParameterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeParameter.
    def visitTypeParameter(self, ctx:CPP14Parser.TypeParameterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#simpleTemplateId.
    def visitSimpleTemplateId(self, ctx:CPP14Parser.SimpleTemplateIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateId.
    def visitTemplateId(self, ctx:CPP14Parser.TemplateIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateName.
    def visitTemplateName(self, ctx:CPP14Parser.TemplateNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateArgumentList.
    def visitTemplateArgumentList(self, ctx:CPP14Parser.TemplateArgumentListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#templateArgument.
    def visitTemplateArgument(self, ctx:CPP14Parser.TemplateArgumentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeNameSpecifier.
    def visitTypeNameSpecifier(self, ctx:CPP14Parser.TypeNameSpecifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#explicitInstantiation.
    def visitExplicitInstantiation(self, ctx:CPP14Parser.ExplicitInstantiationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#explicitSpecialization.
    def visitExplicitSpecialization(self, ctx:CPP14Parser.ExplicitSpecializationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#tryBlock.
    def visitTryBlock(self, ctx:CPP14Parser.TryBlockContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#functionTryBlock.
    def visitFunctionTryBlock(self, ctx:CPP14Parser.FunctionTryBlockContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#handlerSeq.
    def visitHandlerSeq(self, ctx:CPP14Parser.HandlerSeqContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#handler.
    def visitHandler(self, ctx:CPP14Parser.HandlerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#exceptionDeclaration.
    def visitExceptionDeclaration(self, ctx:CPP14Parser.ExceptionDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#throwExpression.
    def visitThrowExpression(self, ctx:CPP14Parser.ThrowExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#exceptionSpecification.
    def visitExceptionSpecification(self, ctx:CPP14Parser.ExceptionSpecificationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#dynamicExceptionSpecification.
    def visitDynamicExceptionSpecification(self, ctx:CPP14Parser.DynamicExceptionSpecificationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#typeIdList.
    def visitTypeIdList(self, ctx:CPP14Parser.TypeIdListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#noeExceptSpecification.
    def visitNoeExceptSpecification(self, ctx:CPP14Parser.NoeExceptSpecificationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#define.
    def visitDefine(self, ctx:CPP14Parser.DefineContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#theOperator.
    def visitTheOperator(self, ctx:CPP14Parser.TheOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CPP14Parser#literal.
    def visitLiteral(self, ctx:CPP14Parser.LiteralContext):
        return self.visitChildren(ctx)



del CPP14Parser


from importlib.resources import files
import re
from pathlib import Path


def fill(
    replacements: list[str],
    package: str,
    resource: str,
    out_path: Path,
    placeholder: str,
) -> None:
    frame = files(package).joinpath(resource).read_text()
    filled_frame = re.sub(placeholder, lambda m, r=iter(replacements): next(r), frame)
    out_path.write_text(filled_frame)



# Generated from SystemVerilogPreParser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .SystemVerilogPreParser import SystemVerilogPreParser
else:
    from SystemVerilogPreParser import SystemVerilogPreParser

# This class defines a complete listener for a parse tree produced by SystemVerilogPreParser.
class SystemVerilogPreParserListener(ParseTreeListener):

    # Enter a parse tree produced by SystemVerilogPreParser#source_text.
    def enterSource_text(self, ctx:SystemVerilogPreParser.Source_textContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#source_text.
    def exitSource_text(self, ctx:SystemVerilogPreParser.Source_textContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#compiler_directive.
    def enterCompiler_directive(self, ctx:SystemVerilogPreParser.Compiler_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#compiler_directive.
    def exitCompiler_directive(self, ctx:SystemVerilogPreParser.Compiler_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#begin_keywords_directive.
    def enterBegin_keywords_directive(self, ctx:SystemVerilogPreParser.Begin_keywords_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#begin_keywords_directive.
    def exitBegin_keywords_directive(self, ctx:SystemVerilogPreParser.Begin_keywords_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#celldefine_directive.
    def enterCelldefine_directive(self, ctx:SystemVerilogPreParser.Celldefine_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#celldefine_directive.
    def exitCelldefine_directive(self, ctx:SystemVerilogPreParser.Celldefine_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#default_nettype_directive.
    def enterDefault_nettype_directive(self, ctx:SystemVerilogPreParser.Default_nettype_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#default_nettype_directive.
    def exitDefault_nettype_directive(self, ctx:SystemVerilogPreParser.Default_nettype_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#default_nettype_value.
    def enterDefault_nettype_value(self, ctx:SystemVerilogPreParser.Default_nettype_valueContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#default_nettype_value.
    def exitDefault_nettype_value(self, ctx:SystemVerilogPreParser.Default_nettype_valueContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#else_directive.
    def enterElse_directive(self, ctx:SystemVerilogPreParser.Else_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#else_directive.
    def exitElse_directive(self, ctx:SystemVerilogPreParser.Else_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#elsif_directive.
    def enterElsif_directive(self, ctx:SystemVerilogPreParser.Elsif_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#elsif_directive.
    def exitElsif_directive(self, ctx:SystemVerilogPreParser.Elsif_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#end_keywords_directive.
    def enterEnd_keywords_directive(self, ctx:SystemVerilogPreParser.End_keywords_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#end_keywords_directive.
    def exitEnd_keywords_directive(self, ctx:SystemVerilogPreParser.End_keywords_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#endcelldefine_directive.
    def enterEndcelldefine_directive(self, ctx:SystemVerilogPreParser.Endcelldefine_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#endcelldefine_directive.
    def exitEndcelldefine_directive(self, ctx:SystemVerilogPreParser.Endcelldefine_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#endif_directive.
    def enterEndif_directive(self, ctx:SystemVerilogPreParser.Endif_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#endif_directive.
    def exitEndif_directive(self, ctx:SystemVerilogPreParser.Endif_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#file_directive.
    def enterFile_directive(self, ctx:SystemVerilogPreParser.File_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#file_directive.
    def exitFile_directive(self, ctx:SystemVerilogPreParser.File_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#filename.
    def enterFilename(self, ctx:SystemVerilogPreParser.FilenameContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#filename.
    def exitFilename(self, ctx:SystemVerilogPreParser.FilenameContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#group_of_lines.
    def enterGroup_of_lines(self, ctx:SystemVerilogPreParser.Group_of_linesContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#group_of_lines.
    def exitGroup_of_lines(self, ctx:SystemVerilogPreParser.Group_of_linesContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#identifier.
    def enterIdentifier(self, ctx:SystemVerilogPreParser.IdentifierContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#identifier.
    def exitIdentifier(self, ctx:SystemVerilogPreParser.IdentifierContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#ifdef_directive.
    def enterIfdef_directive(self, ctx:SystemVerilogPreParser.Ifdef_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#ifdef_directive.
    def exitIfdef_directive(self, ctx:SystemVerilogPreParser.Ifdef_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#ifndef_directive.
    def enterIfndef_directive(self, ctx:SystemVerilogPreParser.Ifndef_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#ifndef_directive.
    def exitIfndef_directive(self, ctx:SystemVerilogPreParser.Ifndef_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#include_directive.
    def enterInclude_directive(self, ctx:SystemVerilogPreParser.Include_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#include_directive.
    def exitInclude_directive(self, ctx:SystemVerilogPreParser.Include_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#level.
    def enterLevel(self, ctx:SystemVerilogPreParser.LevelContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#level.
    def exitLevel(self, ctx:SystemVerilogPreParser.LevelContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#line_directive.
    def enterLine_directive(self, ctx:SystemVerilogPreParser.Line_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#line_directive.
    def exitLine_directive(self, ctx:SystemVerilogPreParser.Line_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#line_directive_.
    def enterLine_directive_(self, ctx:SystemVerilogPreParser.Line_directive_Context):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#line_directive_.
    def exitLine_directive_(self, ctx:SystemVerilogPreParser.Line_directive_Context):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_delimiter.
    def enterMacro_delimiter(self, ctx:SystemVerilogPreParser.Macro_delimiterContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_delimiter.
    def exitMacro_delimiter(self, ctx:SystemVerilogPreParser.Macro_delimiterContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_esc_newline.
    def enterMacro_esc_newline(self, ctx:SystemVerilogPreParser.Macro_esc_newlineContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_esc_newline.
    def exitMacro_esc_newline(self, ctx:SystemVerilogPreParser.Macro_esc_newlineContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_esc_quote.
    def enterMacro_esc_quote(self, ctx:SystemVerilogPreParser.Macro_esc_quoteContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_esc_quote.
    def exitMacro_esc_quote(self, ctx:SystemVerilogPreParser.Macro_esc_quoteContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_identifier.
    def enterMacro_identifier(self, ctx:SystemVerilogPreParser.Macro_identifierContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_identifier.
    def exitMacro_identifier(self, ctx:SystemVerilogPreParser.Macro_identifierContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_name.
    def enterMacro_name(self, ctx:SystemVerilogPreParser.Macro_nameContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_name.
    def exitMacro_name(self, ctx:SystemVerilogPreParser.Macro_nameContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_quote.
    def enterMacro_quote(self, ctx:SystemVerilogPreParser.Macro_quoteContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_quote.
    def exitMacro_quote(self, ctx:SystemVerilogPreParser.Macro_quoteContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_text.
    def enterMacro_text(self, ctx:SystemVerilogPreParser.Macro_textContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_text.
    def exitMacro_text(self, ctx:SystemVerilogPreParser.Macro_textContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#macro_usage.
    def enterMacro_usage(self, ctx:SystemVerilogPreParser.Macro_usageContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#macro_usage.
    def exitMacro_usage(self, ctx:SystemVerilogPreParser.Macro_usageContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#nounconnected_drive_directive.
    def enterNounconnected_drive_directive(self, ctx:SystemVerilogPreParser.Nounconnected_drive_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#nounconnected_drive_directive.
    def exitNounconnected_drive_directive(self, ctx:SystemVerilogPreParser.Nounconnected_drive_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#number.
    def enterNumber(self, ctx:SystemVerilogPreParser.NumberContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#number.
    def exitNumber(self, ctx:SystemVerilogPreParser.NumberContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#pragma_directive.
    def enterPragma_directive(self, ctx:SystemVerilogPreParser.Pragma_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#pragma_directive.
    def exitPragma_directive(self, ctx:SystemVerilogPreParser.Pragma_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#pragma_expression.
    def enterPragma_expression(self, ctx:SystemVerilogPreParser.Pragma_expressionContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#pragma_expression.
    def exitPragma_expression(self, ctx:SystemVerilogPreParser.Pragma_expressionContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#pragma_keyword.
    def enterPragma_keyword(self, ctx:SystemVerilogPreParser.Pragma_keywordContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#pragma_keyword.
    def exitPragma_keyword(self, ctx:SystemVerilogPreParser.Pragma_keywordContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#pragma_name.
    def enterPragma_name(self, ctx:SystemVerilogPreParser.Pragma_nameContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#pragma_name.
    def exitPragma_name(self, ctx:SystemVerilogPreParser.Pragma_nameContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#pragma_value.
    def enterPragma_value(self, ctx:SystemVerilogPreParser.Pragma_valueContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#pragma_value.
    def exitPragma_value(self, ctx:SystemVerilogPreParser.Pragma_valueContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#resetall_directive.
    def enterResetall_directive(self, ctx:SystemVerilogPreParser.Resetall_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#resetall_directive.
    def exitResetall_directive(self, ctx:SystemVerilogPreParser.Resetall_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#source_text_.
    def enterSource_text_(self, ctx:SystemVerilogPreParser.Source_text_Context):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#source_text_.
    def exitSource_text_(self, ctx:SystemVerilogPreParser.Source_text_Context):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#string_literal.
    def enterString_literal(self, ctx:SystemVerilogPreParser.String_literalContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#string_literal.
    def exitString_literal(self, ctx:SystemVerilogPreParser.String_literalContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#text_macro_definition.
    def enterText_macro_definition(self, ctx:SystemVerilogPreParser.Text_macro_definitionContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#text_macro_definition.
    def exitText_macro_definition(self, ctx:SystemVerilogPreParser.Text_macro_definitionContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#text_macro_usage.
    def enterText_macro_usage(self, ctx:SystemVerilogPreParser.Text_macro_usageContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#text_macro_usage.
    def exitText_macro_usage(self, ctx:SystemVerilogPreParser.Text_macro_usageContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#time_precision.
    def enterTime_precision(self, ctx:SystemVerilogPreParser.Time_precisionContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#time_precision.
    def exitTime_precision(self, ctx:SystemVerilogPreParser.Time_precisionContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#time_unit.
    def enterTime_unit(self, ctx:SystemVerilogPreParser.Time_unitContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#time_unit.
    def exitTime_unit(self, ctx:SystemVerilogPreParser.Time_unitContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#timescale_directive.
    def enterTimescale_directive(self, ctx:SystemVerilogPreParser.Timescale_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#timescale_directive.
    def exitTimescale_directive(self, ctx:SystemVerilogPreParser.Timescale_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#unconnected_drive_directive.
    def enterUnconnected_drive_directive(self, ctx:SystemVerilogPreParser.Unconnected_drive_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#unconnected_drive_directive.
    def exitUnconnected_drive_directive(self, ctx:SystemVerilogPreParser.Unconnected_drive_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#unconnected_drive_value.
    def enterUnconnected_drive_value(self, ctx:SystemVerilogPreParser.Unconnected_drive_valueContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#unconnected_drive_value.
    def exitUnconnected_drive_value(self, ctx:SystemVerilogPreParser.Unconnected_drive_valueContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#undef_directive.
    def enterUndef_directive(self, ctx:SystemVerilogPreParser.Undef_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#undef_directive.
    def exitUndef_directive(self, ctx:SystemVerilogPreParser.Undef_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#undefineall_directive.
    def enterUndefineall_directive(self, ctx:SystemVerilogPreParser.Undefineall_directiveContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#undefineall_directive.
    def exitUndefineall_directive(self, ctx:SystemVerilogPreParser.Undefineall_directiveContext):
        pass


    # Enter a parse tree produced by SystemVerilogPreParser#version_specifier.
    def enterVersion_specifier(self, ctx:SystemVerilogPreParser.Version_specifierContext):
        pass

    # Exit a parse tree produced by SystemVerilogPreParser#version_specifier.
    def exitVersion_specifier(self, ctx:SystemVerilogPreParser.Version_specifierContext):
        pass



del SystemVerilogPreParser


from pathlib import Path

from antlr4.tree.Tree import ParseTree
from dovado_rtl.parsers.utilities import HdlAntlrParser
from dovado_rtl.parsers.verilog.verilog_parsed import VerilogParsed
from dovado_rtl.parsers.verilog.generated.VerilogLexer import (
    VerilogLexer as GeneratedVerilog2001Lexer,
)
from dovado_rtl.parsers.verilog.generated.VerilogParser import (
    VerilogParser as GeneratedVerilog2001Parser,
)
from dovado_rtl.parsers.verilog.verilog_visitor import (
    VerilogVisitor,
)

import antlr4


class VerilogParser(HdlAntlrParser):
    __grammar_top_rule: str = "source_text"

    def parse(self, to_parse: Path) -> VerilogParsed:
        token_stream, modules = self._parse(to_parse)
        return VerilogParsed(token_stream, modules)

    @property
    def _grammar_top_rule(self) -> str:
        return self.__grammar_top_rule

    @property
    def _lexer_type(self) -> type[antlr4.Lexer]:
        return GeneratedVerilog2001Lexer

    @property
    def _parser_type(self) -> type[antlr4.Parser]:
        return GeneratedVerilog2001Parser

    @property
    def _visitor_type(self) -> type[ParseTree]:
        return VerilogVisitor
